/*
*       Copyright (c) 2011-2021 by esd electronics gmbh
*
*  This software is copyrighted by and is the sole property of
*  esd gmbh.  All rights, title, ownership, or other interests
*  in the software remain the property of esd gmbh. This
*  software may only be used in accordance with the corresponding
*  license agreement.  Any unauthorized use, duplication, transmission,
*  distribution, or disclosure of this software is expressly forbidden.
*
*  This Copyright notice may not be removed or modified without prior
*  written consent of esd gmbh.
*
*  esd gmbh, reserves the right to modify this software without notice.
*
*  esd electronics gmbh                   Tel. +49-511-37298-0
*  Vahrenwalder Strasse 207               Fax. +49-511-37298-68
*  30165 Hannover                         http://www.esd.eu
*  Germany                                sales@esd.eu
*
*************************************************************************
*
* Description:
* =============================================================================
* File is *INCLUDED* by essHAL.c (to use same code for ECS-xxx/FPGA based HALs)
*
* Offers:
*  ecs_xmc_fpga_ioctl() and ecs_xmc_fpga_ioctl_getdrvinfostr()
*
*/

#ifdef _MSC_VER
# define MY_GMTIME(pSrc, pDst)          {struct tm *_tm = gmtime(pSrc); if(_tm) *pDst = *_tm;}
#else
# define MY_GMTIME(pSrc, pDst)          gmtime_r((pSrc), (pDst));
#endif

#ifdef _MSC_VER
# define MY_SNPRINTF(x)                 _snprintf x
#else
# define MY_SNPRINTF(x)                 snprintf x
#endif

static ESS_RESULT ecs_xmc_fpga_ioctl_getdrvinfostr(DEV_INFOS* dev, const char* base, void* data, uint32_t sizeOfData)
{
    const uint32_t verinfo = SWAP32_LE2HOST(*((const uint32_t*)dev->pSysId));
    const time_t builddate = (time_t)SWAP32_LE2HOST(*((const uint32_t*)dev->pSysId + 1));
    const uint8_t verinfo_major = (uint8_t)(verinfo >> 24);
    const uint16_t verinfo_minor = (uint16_t)((verinfo >> 12) & 0xfff);
    const uint16_t verinfo_revision = (uint16_t)(verinfo & 0xfff);
    const uint16_t pioXMC = SWAP16_LE2HOST(*((const uint16_t*)dev->pPIO_XMC));
    const uint16_t pioPMC = SWAP16_LE2HOST(*((const uint16_t*)dev->pPIO_PMC));
    const uint32_t pioHWVer = *((const uint8_t*)dev->pPIORevision) & 0x07;
    const uint32_t pioFeat = (*((const uint8_t*)dev->pPIORevision) >> 3) & 0x1f;

    char bufDate[32];
    struct tm tm;
    int len;
    memset(&tm, 0, sizeof(struct tm));
    tm.tm_mday = 1;
    MY_GMTIME(&builddate, &tm);
    (void)strftime(bufDate, sizeof(bufDate), "%Y-%m-%d %H:%M:%S", &tm);

    len = MY_SNPRINTF((data, sizeOfData, "%s\nFPGA V%u.%u.%u (built at %s), PIO: HW=%u Feat=0x%.2x I/O=0x%.4x/0x%.4x", base, 
    		verinfo_major, verinfo_minor, verinfo_revision, bufDate, pioHWVer, pioFeat, pioXMC, pioPMC));

    return (len < (int)sizeOfData) ? ESS_RESULT_SUCCESS : ESS_RESULT_INSUFFICIENT_BUFFER;
}

static ESS_RESULT ecs_xmc_fpga_ioctl(DEV_INFOS* dev, ESS_IOCTL fn, void* data, uint32_t sizeOfData, int* handled)
{
    *handled = 1;
    switch (fn) {
        case ESS_IOCTL_GET_DEV_TEMP16:
            {
                const uint16_t val = ecs_xmc_fpga_get_temperature(dev->pI2C);

                if ((data == NULL) || (sizeOfData < sizeof(uint16_t)))
                    return ESS_RESULT_INSUFFICIENT_BUFFER;

                memcpy(data, &val, sizeof(val));
                if ((val == 0) || (val == 0xffff))
                    return ESS_RESULT_UNKNOWN;

                return ESS_RESULT_SUCCESS;
            }

        case ESS_IOCTL_GET_DEV_PRODNAMESTR:
            if (dev->configDataRead == 0)
                dev->configDataRead = ecs_xmc_fpga_flash_readconfigdata(dev->pFlash, &dev->configData);

            if (dev->configDataRead == 1) {
                const size_t srcLen = strnlen(dev->configData.prodId, sizeof(dev->configData.prodId));

                if ((data == NULL) || (sizeOfData < srcLen + 1))
                    return ESS_RESULT_INSUFFICIENT_BUFFER;

                memcpy(data, dev->configData.prodId, srcLen);
                ((char*)data)[srcLen] = '\0';

                return ESS_RESULT_SUCCESS;

            } else if (dev->configDataRead == -2) {
                return ESS_RESULT_INVALID_CONFIG;

            } else {
                return ESS_RESULT_UNKNOWN;
            }

        case ESS_IOCTL_GET_DEV_ORDERNOSTR:
            if (dev->configDataRead == 0)
                dev->configDataRead = ecs_xmc_fpga_flash_readconfigdata(dev->pFlash, &dev->configData);

            if (dev->configDataRead == 1) {
                const size_t srcLen = strnlen(dev->configData.orderNo, sizeof(dev->configData.orderNo));

                if ((data == NULL) || (sizeOfData < srcLen + 1))
                    return ESS_RESULT_INSUFFICIENT_BUFFER;

                memcpy(data, dev->configData.orderNo, srcLen);
                ((char*)data)[srcLen] = '\0';

                return ESS_RESULT_SUCCESS;

            } else if (dev->configDataRead == -2) {
                return ESS_RESULT_INVALID_CONFIG;
            } else {
                return ESS_RESULT_UNKNOWN;
            }

        case ESS_IOCTL_GET_DEV_SERIALNO32:
            if (dev->configDataRead == 0)
                dev->configDataRead = ecs_xmc_fpga_flash_readconfigdata(dev->pFlash, &dev->configData);

            if (dev->configDataRead == 1) {
                if ((data == NULL) || (sizeOfData < sizeof(dev->configData.serial)))
                    return ESS_RESULT_INSUFFICIENT_BUFFER;

                memcpy(data, &dev->configData.serial, sizeof(dev->configData.serial));
                return ESS_RESULT_SUCCESS;

            } else if (dev->configDataRead == -2) {
                return ESS_RESULT_INVALID_CONFIG;
            } else {
                return ESS_RESULT_UNKNOWN;
            }

        case ESS_IOCTL_GET_DEV_HWVERSION32:
            {
                const uint32_t val = *((const uint8_t*)dev->pPIORevision) & 0x07;

                if ((data == NULL) || (sizeOfData < sizeof(uint32_t)))
                    return ESS_RESULT_INSUFFICIENT_BUFFER;

                memcpy(data, &val, sizeof(val));

                return ESS_RESULT_SUCCESS;
            }

        case ESS_IOCTL_GET_DEV_HWFEATURES32:
            {
                const uint32_t val = (*((const uint8_t*)dev->pPIORevision) >> 3) & 0x1f;

                if ((data == NULL) || (sizeOfData < sizeof(uint32_t)))
                    return ESS_RESULT_INSUFFICIENT_BUFFER;

                memcpy(data, &val, sizeof(val));

                return ESS_RESULT_SUCCESS;
            }

        case ESS_IOCTL_DEV_RAWREAD:
            {
                ESS_IOCTL_RAW_DATA* const rd = (ESS_IOCTL_RAW_DATA*)data;
                int res;

                if ((data == NULL) || (sizeOfData < sizeof(ESS_IOCTL_RAW_DATA)) || (rd->data == NULL))
                    return ESS_RESULT_INVALID_ARG;

                if (rd->target != 0x30697073)
                    return ESS_RESULT_INVALID_ARG;

                res = ecs_xmc_fpga_flash_read(dev->pFlash, rd->offset, rd->data, rd->length);

                return (res) ? ESS_RESULT_UNKNOWN : ESS_RESULT_SUCCESS;
            }

        case ESS_IOCTL_DEV_RAWWRITE:
            {
                ESS_IOCTL_RAW_DATA* const rd = (ESS_IOCTL_RAW_DATA*)data;
                int res;

                if ((data == NULL) || (sizeOfData < sizeof(ESS_IOCTL_RAW_DATA)) || (rd->data == NULL))
                    return ESS_RESULT_INVALID_ARG;

                if (rd->target != 0x30697073)
                    return ESS_RESULT_INVALID_ARG;

                res = ecs_xmc_fpga_flash_write(dev->pFlash, rd->offset, rd->data, rd->length);

                return (res) ? ESS_RESULT_UNKNOWN : ESS_RESULT_SUCCESS;
            }

        case ESS_IOCTL_DEV_GPIO:
            {
                ESS_IOCTL_GPIO_DATA* const gd = (ESS_IOCTL_GPIO_DATA*)data;
                volatile uint32_t *pGPIO;
                switch(gd->bankselect) {
                    case 0:
                        pGPIO=dev->pPIO_XMC;
                        break;
                    case 1:
                        pGPIO=dev->pPIO_PMC;
                        break;
                    default:
                        return ESS_RESULT_INVALID_ARG;
                }
                gd->input=pGPIO[0]; /* data */
                pGPIO[5]=(~gd->output) & gd->output_mask;  /* outclear */
                pGPIO[4]=gd->output & gd->output_mask;     /* outset */
                pGPIO[1]=(pGPIO[1] & (~gd->direction_mask)) | (gd->direction & gd->direction_mask); /* direction */
                return ESS_RESULT_SUCCESS;
            }
        default:
            *handled = 0;
            return ESS_RESULT_NOT_IMPLEMENTED;
    }
}
