/*
 * File is *INCLUDED* by HAL implementation for the esd FPGA
 *                    based EtherCAT interfaces
 *
 * Exports:
 *  uint16_t ecs_xmc_fpga_get_temperature(void* pI2C);
 *
 */

#define TMP100_ADDR             0x48
#define TMP100_I2C_FREQ         100000
#define CORE_FREQ               (100 * 1000 * 1000)

#if CFG_BIG_ENDIAN
static uint32_t myLongSwap(uint32_t x);
#define IORD(p, offs)           myLongSwap(*((volatile const uint32_t*)(p) + (offs))) 
#define IOWR(p, offs, data)     do { *((volatile uint32_t*)(p) + (offs)) = (myLongSwap(data)); } while (0)

/* separate function required since SWAPXXX can be used on volatile data */ 
static uint32_t myLongSwap(uint32_t x)
{
	return LONGSWAP(x);
}
#else
#define IORD(p, offs)           (*((volatile const uint32_t*)(p) + (offs)))
#define IOWR(p, offs, data)     do { *((volatile uint32_t*)(p) + (offs)) = (data); } while (0)
#endif

#if 0
# define I2C_DEBUG
#endif

#if 1 /* All this based on opencore samples (i2c_opencores.c / I2C_tests.c) */

#define IOADDR_I2C_OPENCORES_PRERLO(base)       __IO_CALC_ADDRESS_NATIVE(base, 0)
#define IORD_I2C_OPENCORES_PRERLO(base)         IORD(base, 0)
#define IOWR_I2C_OPENCORES_PRERLO(base, data)   IOWR(base, 0, data)

#define IOADDR_I2C_OPENCORES_PRERHI(base)       __IO_CALC_ADDRESS_NATIVE(base, 0)
#define IORD_I2C_OPENCORES_PRERHI(base)         IORD(base, 1)
#define IOWR_I2C_OPENCORES_PRERHI(base, data)   IOWR(base, 1, data)

#define IOADDR_I2C_OPENCORES_CTR(base)          __IO_CALC_ADDRESS_NATIVE(base, 2)
#define IORD_I2C_OPENCORES_CTR(base)            IORD(base, 2)
#define IOWR_I2C_OPENCORES_CTR(base, data)      IOWR(base, 2, data)
/* bit definitions*/
#define I2C_OPENCORES_CTR_EN_MSK                (0x80)
#define I2C_OPENCORES_CTR_EN_OFST               (7)
#define I2C_OPENCORES_CTR_IEN_MSK               (0x40)
#define I2C_OPENCORES_CTR_IEN_OFST              (6)

#define IOADDR_I2C_OPENCORES_TXR(base)          __IO_CALC_ADDRESS_NATIVE(base, 3)
#define IOWR_I2C_OPENCORES_TXR(base, data)      IOWR(base, 3, data)
/* bit definitions*/
#define I2C_OPENCORES_TXR_RD_MSK                (0x1)
#define I2C_OPENCORES_TXR_RD_OFST               (0)
#define I2C_OPENCORES_TXR_WR_MSK                (0x0)
#define I2C_OPENCORES_TXR_WR_OFST               (0)

#define IOADDR_I2C_OPENCORES_RXR(base)          __IO_CALC_ADDRESS_NATIVE(base, 3)
#define IORD_I2C_OPENCORES_RXR(base)            IORD(base, 3)

#define IOADDR_I2C_OPENCORES_CR(base)           __IO_CALC_ADDRESS_NATIVE(base, 4)
#define IOWR_I2C_OPENCORES_CR(base, data)       IOWR(base, 4, data)
/* bit definitions*/
#define I2C_OPENCORES_CR_STA_MSK                (0x80)
#define I2C_OPENCORES_CR_STA_OFST               (7)
#define I2C_OPENCORES_CR_STO_MSK                (0x40)
#define I2C_OPENCORES_CR_STO_OFST               (6)
#define I2C_OPENCORES_CR_RD_MSK                 (0x20)
#define I2C_OPENCORES_CR_RD_OFST                (5)
#define I2C_OPENCORES_CR_WR_MSK                 (0x10)
#define I2C_OPENCORES_CR_WR_OFST                (4)
#define I2C_OPENCORES_CR_NACK_MSK               (0x8)
#define I2C_OPENCORES_CR_NACK_OFST              (3)
#define I2C_OPENCORES_CR_IACK_MSK               (0x1)
#define I2C_OPENCORES_CR_IACK_OFST              (0)

#define IOADDR_I2C_OPENCORES_SR(base)           __IO_CALC_ADDRESS_NATIVE(base, 4)
#define IORD_I2C_OPENCORES_SR(base)             IORD(base, 4)
/* bit definitions*/
#define I2C_OPENCORES_SR_RXNACK_MSK             (0x80)
#define I2C_OPENCORES_SR_RXNACK_OFST            (7)
#define I2C_OPENCORES_SR_BUSY_MSK               (0x40)
#define I2C_OPENCORES_SR_BUSY_OFST              (6)
#define I2C_OPENCORES_SR_AL_MSK                 (0x20)
#define I2C_OPENCORES_SR_AL_OFST                (5)
#define I2C_OPENCORES_SR_TIP_MSK                (0x2)
#define I2C_OPENCORES_SR_TIP_OFST               (1)
#define I2C_OPENCORES_SR_IF_MSK                 (0x1)
#define I2C_OPENCORES_SR_IF_OFST                (0)

/*
void I2C_init(uint32_t base, uint32_t clk, uint32_t speed);
int I2C_start(uint32_t base, uint32_t add, uint32_t read);
int I2C_read(uint32_t base, uint32_t last);
int I2C_write(uint32_t base, uint8_t data, uint32_t last);
*/

#define I2C_OK                                  (0)
#define I2C_ACK                                 (0)
#define I2C_NOACK                               (1)
#define I2C_ABITRATION_LOST                     (2)


/* these functions are polled only.  */
/* all functions wait until the I2C is done before exiting */

/****************************************************************
 int I2C_init
 This function initializes the prescalor for the scl
 and then enables the core. This must be run before
 any other i2c code is executed
 inputs
 base = the base address of the component
 clk = frequency of the clock driving this component  ( in Hz)
 speed = SCL speed ie 100K, 400K ...            (in Hz)
 15-OCT-07 initial release
 *****************************************************************/
static void I2C_init(void* base, uint32_t clk, uint32_t speed)
{
    uint32_t prescale = (clk / (5 * speed)) - 1;

#ifdef  I2C_DEBUG
    printf(" Initializing  I2C at 0x%x, \n\twith clock speed 0x%x \n\tand SCL speed 0x%x \n\tand prescale 0x%x\n",base,clk,speed,prescale);
#endif

    IOWR_I2C_OPENCORES_CTR(base, 0x00); /* turn off the core*/
    IOWR_I2C_OPENCORES_CR(base, I2C_OPENCORES_CR_IACK_MSK); /* clear any pending IRQ*/
    IOWR_I2C_OPENCORES_PRERLO(base, (0xff & prescale)); /* load low presacle bit*/
    IOWR_I2C_OPENCORES_PRERHI(base, (0xff & (prescale >> 8))); /* load upper prescale bit */
    IOWR_I2C_OPENCORES_CTR(base, I2C_OPENCORES_CTR_EN_MSK); /* turn on the core*/
}

/****************************************************************
 int I2C_start
 Sets the start bit and then sends the first byte which
 is the address of the device + the write bit.
 inputs
 base = the base address of the component
 add = address of I2C device
 read =  1== read    0== write
 return value
 0 if address is acknowledged
 1 if address was not acknowledged
 15-OCT-07 initial release
 *****************************************************************/
static int I2C_start(void* base, uint32_t add, uint32_t isread)
{
#ifdef  I2C_DEBUG
    printf(" Start  I2C at 0x%x, \n\twith address 0x%x \n\tand read 0x%x\n",base,add,read);
#endif

    /* transmit the address shifted by one and the read/write bit*/
    IOWR_I2C_OPENCORES_TXR(base, ((add << 1) + (0x1 & isread)));

    /* set start and write  bits which will start the transaction*/
    IOWR_I2C_OPENCORES_CR(base, I2C_OPENCORES_CR_STA_MSK | I2C_OPENCORES_CR_WR_MSK);

    /* wait for the transaction to be over.*/
    while (IORD_I2C_OPENCORES_SR(base) & I2C_OPENCORES_SR_TIP_MSK)
        ;

    /* now check to see if the address was acknowledged */
    if (IORD_I2C_OPENCORES_SR(base) & I2C_OPENCORES_SR_RXNACK_MSK) {
#ifdef  I2C_DEBUG
        printf("\tNOACK\n");
#endif
        return (I2C_NOACK);
    } else {
#ifdef  I2C_DEBUG
        printf("\tACK\n");
#endif
        return (I2C_ACK);
    }
}

/****************************************************************
 int I2C_read
 assumes that any addressing and start
 has already been done.
 reads one byte of data from the slave.  on the last read
 we don't acknowledge and set the stop bit.
 inputs
 base = the base address of the component
 last = on the last read there must not be a ack

 return value
 byte read back.
 15-OCT-07 initial release
 *****************************************************************/
static int I2C_read(void* base, uint32_t last)
{
#ifdef  I2C_DEBUG
    printf(" Read I2C at 0x%x, \n\twith last0x%x\n",base,last);
#endif

    if (last) {
        /* start a read and no ack and stop bit*/
        IOWR_I2C_OPENCORES_CR(base, I2C_OPENCORES_CR_RD_MSK | I2C_OPENCORES_CR_NACK_MSK | I2C_OPENCORES_CR_STO_MSK);
    } else {
        /* start read*/
        IOWR_I2C_OPENCORES_CR(base, I2C_OPENCORES_CR_RD_MSK);
    }

    /* wait for the transaction to be over.*/
    while (IORD_I2C_OPENCORES_SR(base) & I2C_OPENCORES_SR_TIP_MSK)
        ;

    /* now read the data */
    return ((int)IORD_I2C_OPENCORES_RXR(base));
}

/****************************************************************
 int I2C_write
 assumes that any addressing and start
 has already been done.
 writes one byte of data from the slave.
 If last is set the stop bit set.
 inputs
 base = the base address of the component
 data = byte to write
 last = on the last read there must not be a ack

 return value
 0 if address is acknowledged
 1 if address was not acknowledged
 15-OCT-07 initial release
 *****************************************************************/
static int I2C_write(void* base, uint8_t data, uint32_t last)
{
#ifdef  I2C_DEBUG
    printf(" Write I2C at 0x%x, \n\twith data 0x%x,\n\twith last0x%x\n",base,data,last);
#endif
    /* transmit the data*/
    IOWR_I2C_OPENCORES_TXR(base, data);

    if (last) {
        /* start a read and no ack and stop bit*/
        IOWR_I2C_OPENCORES_CR(base, I2C_OPENCORES_CR_WR_MSK | I2C_OPENCORES_CR_STO_MSK);
    } else {
        /* start read*/
        IOWR_I2C_OPENCORES_CR(base, I2C_OPENCORES_CR_WR_MSK);
    }

    /* wait for the transaction to be over.*/
    while (IORD_I2C_OPENCORES_SR(base) & I2C_OPENCORES_SR_TIP_MSK)
        ;

    /* now check to see if the address was acknowledged */
    if (IORD_I2C_OPENCORES_SR(base) & I2C_OPENCORES_SR_RXNACK_MSK) {
#ifdef  I2C_DEBUG
        printf("\tNOACK\n");
#endif
        return (I2C_NOACK);
    } else {
#ifdef  I2C_DEBUG
        printf("\tACK\n");
#endif
        return (I2C_ACK);
    }
}

#endif /* All this based on opencore samples (i2c_opencores.c / I2C_tests.c) */


/* Value from TMP100, in 16th of degrees C. (0 on error) */
static uint16_t ecs_xmc_fpga_get_temperature(void* pI2C)
{
    uint8_t data[2];
    int err;


    I2C_init(pI2C, CORE_FREQ, TMP100_I2C_FREQ);

    /* Address TMP100: (in write mode) */
    err = I2C_start(pI2C, TMP100_ADDR, 0);
    if (err) {
        PRINT(("I2C_start() returned %d!\n", err));
        return 0xffff;
    }
    /* Write 0x00 to pointer register: */
    err = I2C_write(pI2C, 0x00, 1);
    if (err) {
        PRINT(("I2C_write() returned %d!\n", err));
        return 0xffff;
    }

    /* Address TMP100: (in read mode) */
    err = I2C_start(pI2C, TMP100_ADDR, 1);
    if (err) {
        PRINT(("I2C_start() returned %d!\n", err));
        return 0xffff;
    }

    /* Read two bytes: */
    data[0] = (uint8_t)I2C_read(pI2C, 0);
    data[1] = (uint8_t)I2C_read(pI2C, 1); /* 1 = is last read */

    return (data[0] << 4) | (data[1] >> 4);
}
