#!/bin/sh

up_interfaces=

update_network_config() {
	if [ ! -f "/mnt/settings/config/network" ]; then
		# network does not exist, copy old settings in this file
		grep network /mnt/settings/config/genseries | sed 's/genseries.network/network.lan/' >/mnt/settings/config/network
	fi
}




detect_up_interface() {
	local ifc=$1
	config_get up "$ifc" up
	config_get ifname "$ifc" ifname
	ifconfig "$ifname" &>/dev/null
	local eth_detect=$?
	[ "$eth_detect" = 0 ] && {
	    [ -n "$up" ] || {
#
# Do not check for the comm function. Any connected interface will do.
#	        config_get ifc_function "$ifc" function
			echo "ifc: $ifc, func: $ifc_function"
#			[ "$ifc_function" = "all" ] || [ "$ifc_function" = "com" ]  || [ "$ifc" = "lan" ] && {
		    	echo "ifc: $ifc added."
	            append up_interfaces "$ifc"
#			}
	    }
	}
}

force_up_interface() {
	local ifc=$1
	config_get up "$ifc" up
	config_get ifname "$ifc" ifname
	ifconfig "$ifname" &>/dev/null
	local eth_detect=$?
	[ "$eth_detect" = 0 ] && {
	    [ -n "$up" ] || {
	        config_set "$ifc" function com
	    }
	}
}

detect_cable() {
	local detect=$(ethtool "$1" | grep -o "Link detected: \w" | sed 's/^\w* \w*: //')
	[ "$detect" = "y" ] && {
		return 1
	}

	return 0
}

auto_detect_interface() {

	#
	# TODO better place for this function'
	update_network_config 

	config_load network
	config_foreach detect_up_interface interface

	[ -z "$up_interfaces" ] && {
	    echo "ifc: No comm. enabled interfaces detected!"
	    echo "ifc: Forcing interfaces into communication mode."
	    config_foreach force_up_interface interface
	    config_foreach detect_up_interface interface
	}

	local detected_priority=99
	local detected_ifc=
	local detected_ifname=
	# Sleep 0.2 sec every iteration = 5 times per second
	local timeout_cnt=15

	# Wait until an interface has a cable detected
	while [ -z "$detected_ifc" ]
	do
		logger -t 'eth' "Waiting for ethernet cable..."
		sleep 0.2
		for ifc in $up_interfaces; do
			config_get priority "$ifc" priority
			config_get ifname "$ifc" ifname
			[ -n "$priority" ] && {
				detect_cable "$ifname" || {
					[ $priority -lt $detected_priority ] && {
						detected_priority=$priority
						detected_ifc=$ifc
						detected_ifname=$ifname
					}
				}
			}
		done

		[ -z "$detected_ifc" ] && [ "$timeout_cnt" -gt 0 ] && {
			let timeout_cnt=timeout_cnt-1
#			[ "$timeout_cnt" -eq 0 ] && {
#				/usr/sbin/hbm-hw-frontpanel "\fNo Network Cable\nConnected"
#			}
		}
	done

	[ -n "$detected_ifc" ] && {
		logger -t 'eth' "Ethernet cable detected: $detected_ifc ($detected_ifname)"
		setup_interface "$detected_ifname" "$detected_ifc"
		uci_set_state_keep genseries network interface "$detected_ifc"
	}
}

